@@ -8,7 +8,7 @@
 # Basic Libraries
 from typing import Optional, Callable
 # NDN Imports
-from ndn.encoding import Name
+from ndn.encoding import Name, BinaryStr
 from ndn.security import Keychain
 from ndn.transport.stream_socket import Face
 from ndn.storage import Storage
@@ -27,4 +27,7 @@ class SVSyncShared_Thread(SVSyncBase_Thread):
         super().__init__(groupPrefix, nid, updateCallback, storage, securityOptions, face, keychain)
         self.cacheOthers = cacheOthers
     async def function(self) -> None:
-        self.svs = SVSyncShared(self.app, self.groupPrefix, self.nid, self.missing_callback, self.cacheOthers, self.storage, self.secOptions)
\ No newline at end of file
+        self.svs = SVSyncShared(self.app, self.groupPrefix, self.nid, self.missing_callback, self.cacheOthers, self.storage, self.secOptions)
+    def serveDataPacket(datapkt:BinaryStr) -> None:
+        if self.svs:
+            self.svs.serveDataPacket(datapkt)
\ No newline at end of file
